/******************************************************************************

  Sampling von IR Pulsen - Puls/Pause Verhltnis
  MEGA8, 11.0592 MHz
  FB Traegerfrequenz = 36kHz, Periode = 28us

  Funktion:
  
  16 Bit Timer wird initalisiert & zur Zeitmessung verwendet.
  Ein Timerberlauf whrend der Eingang '1' ist fhrt zum Ende.
  Der Timer wird im externen Interrupt gesetzt / gelesen. 
  Wenn ein Ende erkannt wurde, so werden die Stnde ausgegeben.
  Ein Tick dauert ~723ns

  Durch die Buffersize wird die Anzahl Pulse pro Telegram hier auf
  63 begrenzt. 

******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#include <stdlib.h>
#include <inttypes.h>

#include "irsample.h"


unsigned char  ir_ready;
unsigned char  ir_index;
unsigned short ir_ticks[BUFFERSIZE];   // 2*BUFFERSIZE Bytes


// Prescaler Define
#define PRSC1_SELECT (_BV(CS11))           // timer CLK/8

// Einlesen des aktuellen IR Pegel
#define IRDet bit_is_set(PIND,PD3)



/******************************************************************************

  Interrupthandler fuer Timer1, Ueberlauf je nach Timerinitialisierung 
	0 => ~47ms. 

******************************************************************************/
SIGNAL(SIG_OVERFLOW1)
{
  if (ir_index > 10) {  // Bei einer Pause nach weniger als 10 Pulsen wird von 
                     // einem Fehler ausgegangen. Es trat hier ein Timeout auf.
    ir_ready = 1;
    GICR  &=  ~_BV(INT1);		// disable external int1
    TCCR1B = 0;					    // stop timer1
  } else {
    // Bei Timeout ohne vorher erkannte Flanke alles neu initialisieren.
    // Normalfall, wenn auf IR Pulse gewartet wird...
		ir_init();
	}
}



/******************************************************************************

  Interrupthandler fuer Interrupt 1, an diesem haengt der IR Empfaenger..

******************************************************************************/
SIGNAL(SIG_INTERRUPT1)
{
  static unsigned char ErrorFlag = 0;
  unsigned short counter = TCNT1 - TIMERINITVAL;

  if (ir_index >= BUFFERSIZE) {
    ir_ticks[BUFFERSIZE-1] = 0xffff;
  } else if ((ir_index > 0) || ((ir_index == 0) && IRDet)) {
    // Falls das Flag gesetzt wurde, so ist ein zu kurzer Impuls entdeckt worden. 
    // Nun muss die zweite Flaske des Impulses auch noch ignoriert werden.
    if (!ErrorFlag) {
      if (counter > FILTERTHRESHOLD) {
        // Die erste fallende Flanke ist der Beginn des ganzen 
        // Telegramms, also erst die steigende Flanke auswerten.
        TCNT1  = TIMERINITVAL;                 // Counter zurueck setzen
        ir_ticks[ir_index] = counter;
        ir_index ++;
      } else {
        // Stoerimpuls erkannt.
        ErrorFlag = 1;
      }
    } else {
      // Flag gilt immer nur fr eine Flanke
      ErrorFlag = 0;
    }
  } else {
    // der Timer wird erst bei der ersten fallenden Flanke initialisiert
    TCCR1B |= PRSC1_SELECT;					// start timer1
    TCNT1  = TIMERINITVAL;          // Counter auf 0
  }

  MCUCR |=	_BV(ISC10);           // Int1 aktiv fuer beide Flanken
	MCUCR &= ~_BV(ISC11);
}


/******************************************************************************

  Initialisierung der gesamten Hardware

******************************************************************************/
void ir_init(void)
{
  DDRD  &= ~_BV(PD3);             // PD3 auf Eingang, Pullup aktiv
  PORTD |=  _BV(PD3);
	
  MCUCR &= ~_BV(ISC10);
	MCUCR |=  _BV(ISC11);						// raise int1 on falling edge

  GICR  |=  _BV(INT1);						// enable external int1

  TCCR1B = 0;					            // stop timer1

  TIMSK  |= _BV(TOIE1);					  // enable timer1 Overflow interrupts


  ir_index = 0;
  ir_ready = 0;
}


